"""Sphinx configuration."""  # noqa: INP001
project = "chardetng Python Module"
author = "John Parton"
copyright = "2023, John Parton"  # noqa: A001
extensions = [
    "sphinx_toolbox.more_autodoc",
    "sphinx.ext.autodoc",
    "sphinx.ext.napoleon",
    "numpydoc",
]
autodoc_typehints = "description"
html_theme = "furo"
github_username = "john-parton"
github_repository = "chardetng-py"



"""Functions for dealing with byte strings of unknown encoding."""

from typing import Union

from chardetng_py.detector import EncodingDetector


def detect(
    byte_str: Union[bytes, bytearray],
    *,
    allow_utf8: bool = False,
    tld: Union[bytes, bytearray, None] = None,
) -> str:
    """Detect the encoding of :code:`byte_str`.

    Returned encoding is suitable for use with :code:`str.decode`.

    Parameters
    ----------
    byte_str : :code:`bytes` or :code:`bytearray`
        Input buffer to decode.
    allow_utf8 : :code:`bool`
        If set to :code:`False`, the return value of
        this method won't be :code:`"UTF-8"`. When performing detection
        on :code:`"text/html"` on non-:code:`file:` URLs, Web browsers must pass
        :code:`False`,
        unless the user has taken a specific contextual action to request an
        override. This way, Web developers cannot start depending on UTF-8
        detection. Such reliance would make the Web Platform more brittle.
    tld : :code:`bytes` or :code:`bytearray` or :code:`None`
        If :code:`tld` contains non-ASCII, period, or upper-case letters. The exception
        condition is intentionally limited to signs of failing to extract the
        label correctly, failing to provide it in its Punycode form, and failure
        to lower-case it. Full DNS label validation is intentionally not performed
        to avoid panics when the reality doesn't match the specs.
    """
    encoding_detector = EncodingDetector()
    encoding_detector.feed(byte_str, last=True)

    return encoding_detector.guess(tld=tld, allow_utf8=allow_utf8)



"""Test cases for the chardetng_py module."""
from chardetng_py import decode


def test_decode() -> None:
    """Very basic decode litmus test."""
    assert decode(b"Jakby r\xeaka Boga") == "Jakby rêka Boga"



# noqa: D100
import typing

Buffer: typing.TypeAlias = bytes | bytearray

class EncodingDetector:  # noqa: D101
    def feed(
        self: typing.Self,
        buffer: Buffer,
        /,
        *,
        last: bool,
    ) -> bool: ...
    def guess(
        self: typing.Self,
        *,
        tld: bytes | None,
        allow_utf8: bool,
    ) -> str: ...
    def guess_assess(
        self: typing.Self,
        *,
        tld: bytes | None,
        allow_utf8: bool,
    ) -> tuple[str, bool]: ...



"""Functions to aid in migrating from chardet or charset_normalizer to chardetng_py."""

import sys
from typing import Final, Union

from chardetng_py.shortcuts import detect as _detect

# Older versions of Python have a TypedDict with limited functionality
# This helps documentations tools and type checkers
if sys.version_info >= (3, 11):
    from typing import TypedDict
else:
    from typing_extensions import TypedDict


# chardetng does not return a confidence value
# This is the value which is unconditionally returned
DEFAULT_CONFIDENCE: Final[float] = 0.99


# TypedDict was introduced in Python 3.8.
class ResultDict(TypedDict):
    """Return value for detect compatability function."""

    encoding: str
    confidence: float
    language: None


def detect(byte_str: Union[bytes, bytearray]) -> ResultDict:
    """Detect the encoding of a string and return additional information.

    Detect the encoding of the given byte string. Language detect is not implemented
    and will always return :code:`None`. Confidence is always :code:`0.99`.

    Parameters
    ----------
    byte_str : :code:`bytes` or :code:`bytearray`
        Input buffer to detect the encoding of.
    """
    return {
        "encoding": _detect(byte_str),
        "confidence": DEFAULT_CONFIDENCE,
        # chardetng does not return a language
        "language": None,
    }



"""Functions for dealing with byte strings of unknown encoding.".

:code:`detect` is imported from :code:`chardetng_py.api` and so therefore may
be imported using :code:`from chardetng_py import detect`.
"""

from typing import Final, List

from chardetng_py.detector import EncodingDetector
from chardetng_py.shortcuts import detect

# Documentation for the detect function is on the rust function
# in lib.rs

__all__: Final[List[str]] = [
    "detect",
    "EncodingDetector",
]
